package outils;

import static org.junit.jupiter.api.Assertions.*;

import java.util.Iterator;

import org.junit.jupiter.api.Test;

public class TestListeChainee {
	
    @Test
    public void test01Constructeur() {
        Liste<String> lst;
        lst = new ListeChainee<String>();
        assertTrue(lst.estVide());
        assertEquals(0, lst.nombreDElements());
    }

    @Test
    public void test02Ajout() {
        Liste<String> lst;
        lst = new ListeChainee<String>();
        lst.ajoute("un");
        assertFalse(lst.estVide());
        assertEquals(1, lst.nombreDElements());
        assertEquals("un", lst.element(0));
        lst.ajoute("deux");
        assertEquals(2, lst.nombreDElements());
        assertEquals("deux", lst.element(1));
    }

    @Test
    public void test03Retrait() {
        Liste<String> lst;
        lst = new ListeChainee<String>();
        lst.ajoute("un");
        lst.ajoute("deux");
        lst.ajoute("trois");
        lst.ajoute("quatre");
        lst.retire(1); //retrait au milieu
        assertEquals(3, lst.nombreDElements());
        assertEquals("un", lst.element(0));
        assertEquals("trois", lst.element(1));
        lst.retire(2); //retrait à la fin
        assertEquals(2, lst.nombreDElements());
        assertEquals("un", lst.element(0));
        assertEquals("trois", lst.element(1));
        lst.retire(0); //retrait au début
        assertEquals(1, lst.nombreDElements());
        assertEquals("trois", lst.element(0));
        lst.retire(0); //retrait au début
        assertTrue(lst.estVide());
    }

    @Test
    public void test04Element() {
        Liste<String> lst;
        lst = new ListeChainee<String>();
        lst.ajoute("un");
        lst.ajoute("deux");
        lst.ajoute("trois");
//      assertNull(lst.element(-1));
        assertEquals("un", lst.element(0));
        assertEquals("deux", lst.element(1));
        assertEquals("trois", lst.element(2));
//      assertNull(lst.element(3));
    }

    @Test
    public void test05Parcours() {
        Liste<String> lst;
        lst = new ListeChainee<String>();
        String textes[] = {"un", "deux", "trois", "quatre", "cinq"};
        for(String txt : textes) lst.ajoute(txt);
        Parcours<String> parcours = lst.parcours();
        int i;
        i = 0;
        while(parcours.encore()) {
        	assertEquals(textes[i++], parcours.elementCourant());
        	parcours.suite();
        }
    }

    @Test
    public void test06ParcoursAvecForEach() {
        Liste<String> lst = new ListeChainee<String>();
        String textes[] = {"un", "deux", "trois", "quatre", "cinq"};
        for(String txt : textes) lst.ajoute(txt);
        int i = 0;
        for(String s : lst) assertEquals(textes[i++], s);
    }

    @Test
    public void test07ParcoursAvecSuppression() {
        Liste<String> lst = new ListeChainee<String>();
        String textes[] = {"un", "deux", "trois", "quatre", "cinq"};
        for(String txt : textes) lst.ajoute(txt);
        Parcours<String> parcours = lst.parcours();
        while(parcours.encore()) {
        	String s = parcours.elementCourant();
        	if (s.length() == 4)
        		parcours.supprime();
        	else
        		parcours.suite();
        }
        String textesRestant[] = {"un", "trois", "quatre"};
        int i = 0;
        for(String s : lst) assertEquals(textesRestant[i++], s);
    }

    @Test
    public void test08ParcoursAvecSuppression() {
        Liste<String> lst = new ListeChainee<String>();
        String textes[] = {"un", "deux", "trois", "quatre", "cinq"};
        for(String txt : textes) lst.ajoute(txt);
        Iterator<String> it = lst.iterator();
        while(it.hasNext()) {
        	String s = it.next();
        	if (s.length() == 4)
        		it.remove();
        }
        String textesRestants[] = {"un", "trois", "quatre"};
        int i = 0;
        for(String s : lst) assertEquals(textesRestants[i++], s);
    }
}
